//*****************************************************************************
// File Name            : iobus_2_mig.v
//-----------------------------------------------------------------------------
// Function             : iobus_2_mig 
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo@himwari_co 
//-----------------------------------------------------------------------------
// History
// -.-- 2013/08/02
//*****************************************************************************
module iobus_2_mig (
  reset_n,
  c3_p0_cmd_clk,
  c3_p0_cmd_en,
  c3_p0_cmd_instr,
  c3_p0_cmd_bl,
  c3_p0_cmd_byte_addr,
  c3_p0_cmd_empty,
  c3_p0_cmd_full,
  c3_p0_wr_clk,
  c3_p0_wr_en,
  c3_p0_wr_mask,
  c3_p0_wr_data,
  c3_p0_wr_full,
  c3_p0_wr_empty,
  c3_p0_wr_count,
  c3_p0_wr_underrun,
  c3_p0_wr_error,
  c3_p0_rd_clk,
  c3_p0_rd_en,
  c3_p0_rd_data,
  c3_p0_rd_full,
  c3_p0_rd_empty,
  c3_p0_rd_count,
  c3_p0_rd_overflow,
  c3_p0_rd_error,
  c3_calib_done,     
 //iobus
  Clk,
  Reset,
  IO_Ready,
  IO_Addr_Strobe,
  IO_Read_Strobe,
  IO_Write_Strobe,
  IO_Read_Data,
  IO_Address,
  IO_Byte_Enable,
  IO_Write_Data,
  iobus_mig_sel
);
  //
  //
  input         reset_n;
  //MIG
  output        c3_p0_cmd_clk;
  output        c3_p0_cmd_en;
  output [2:0]  c3_p0_cmd_instr;
  output [5:0]  c3_p0_cmd_bl;
  output [29:0] c3_p0_cmd_byte_addr;
  input         c3_p0_cmd_empty;
  input         c3_p0_cmd_full;
  output        c3_p0_wr_clk;
  output        c3_p0_wr_en;
  output [3:0]  c3_p0_wr_mask;
  output [31:0] c3_p0_wr_data;
  input         c3_p0_wr_full;
  input         c3_p0_wr_empty;
  input [6:0]   c3_p0_wr_count;
  input         c3_p0_wr_underrun;
  input         c3_p0_wr_error;
  output        c3_p0_rd_clk;
  output        c3_p0_rd_en;
  input [31:0]  c3_p0_rd_data;
  input         c3_p0_rd_full;
  input         c3_p0_rd_empty;
  input [6:0]   c3_p0_rd_count;
  input         c3_p0_rd_overflow;
  input         c3_p0_rd_error; 
  input         c3_calib_done ; 
  //io_bus
  input Clk;
  input Reset;
  input IO_Addr_Strobe;
  input IO_Read_Strobe;
  input IO_Write_Strobe;
  input [31 : 0] IO_Address;
  input [3 : 0] IO_Byte_Enable;
  input [31 : 0] IO_Write_Data;
  output [31 : 0] IO_Read_Data;
  output IO_Ready;
  input iobus_mig_sel;
 
  parameter  p_init = 4'b0;   
  parameter  p_idle = 4'b1;   
  parameter  p_dsl  = 4'd2;   
  parameter  p_nop  = 4'd3;   
  parameter  p_pall = 4'd4;   
  parameter  p_init_ref1  = 4'd5;
  parameter  p_init_ref2  = 4'd6;
  parameter  p_mrs  = 4'd7;
  parameter  p_rd_act  = 4'd8;
  parameter  p_rd_wait  = 4'd9;
  parameter  p_rd1  = 4'd10;
  //parameter  p_rd2  = 4'd11;
  parameter  p_wr_act  = 4'd11;
  parameter  p_wr1  = 4'd12;
  //parameter  p_wr_i  = 4'd13;
  //parameter  p_wr2  = 4'd14;
  parameter  p_ref  = 4'd13;
  parameter  t_clear = 16'd0;
  parameter  t_inc   = 16'd1;
  //parameter  t_init_end = 16'd35000;
  //parameter  t_init_end = 16'd35000;  //210us
  parameter  t_init_end = 16'd1000;  //210us
  parameter  t_pall_end = 16'd5;
  parameter  t_ref_end = 16'd11;
  //parameter  t_ref_end = 16'd60;
  parameter  t_mrs_end = 16'd2;
  parameter  t_wr_act_end = 16'd2;
  parameter  t_rd_act_end = 16'd0;
  parameter  t_rd_wait_end = 16'd1000;
  //parameter  t_rd1_end = 16'd12;
  parameter  t_rd1_end = 16'd9;
  //parameter  t_rd_i_end = 16'd2;
  //parameter  t_rd2_end = 16'd12;
  //parameter  t_wr1_end = 16'd7;
  parameter  t_wr1_end = 16'd4;
  //parameter  t_wr_i_end = 16'd2;
  //parameter  t_wr2_end = 16'd7;
  parameter  t_ref_time = 16'd10000;
  //parameter  t_ref_time = 16'd100;
   
  parameter  t_rd_da_en_on = 16'd5;
  parameter  t_rd_da_en_off = 16'd12;
  parameter  t_act_wr_da_en_on = 16'd0;
  parameter  t_wr_da_en_off = 16'd1;
  parameter  t_wr_dqm_start = 16'd2; //state=wr_act
  parameter  t_wr_dqm_end = 16'd2;   //state=wr1
  parameter  t_wr_cmd = 16'd3;   //state=wr1
  parameter  t_rd_dqm_start = 16'd0;
  //parameter  t_rd_dqm_end = 16'd7;
  parameter  t_rd_dqm_end = 16'd0;
   
  reg [3:0]   state;
  reg [15:0]  waite_cnt;
  reg [15:0]  ref_cnt;
   
  reg   c3_p0_cmd_en;
  reg [2:0]     c3_p0_cmd_instr;
  reg [5:0]     c3_p0_cmd_bl;
  reg [29:0]    c3_p0_cmd_byte_addr;
  reg   c3_p0_wr_en;
  reg [3:0]     c3_p0_wr_mask;
  reg [31:0]    c3_p0_wr_data;
  reg   c3_p0_rd_en;
                
  reg [31:0]r_dat_o; //user read  data
  reg  ack_o;
//
reg [31:0] IO_Read_Data;
reg IO_Ready;
   
reg         cyc_o;         // cycle
reg         stb_o;         // strobe
reg  [31:0]  adr_o;         // address adr_o[1]
reg  [3:0]  sel_o;         // byte_sel
reg         we_o;          // write enable
reg  [31:0] dat_o;         // data output
   
//
//IOBUS
always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    cyc_o <= 1'b0;
  else 
    if ((iobus_mig_sel==1'b1)&&((IO_Read_Strobe==1'b1)||(IO_Write_Strobe==1'b1)))
       cyc_o <=  1'b1 ;
    else if (ack_o==1'b1)
       cyc_o <=  1'b0 ;
    else
       cyc_o <=  cyc_o ;
    
always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    stb_o <= 1'b0;
  else 
    if  ((iobus_mig_sel==1'b1)&&((IO_Read_Strobe==1'b1)||(IO_Write_Strobe==1'b1)))
       stb_o <=  1'b1 ;
    else if  (ack_o==1'b1)
       stb_o <=  1'b0 ;
    else
       stb_o <=  stb_o ;
   
always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    we_o <= 1'b0;
  else 
    if ((iobus_mig_sel==1'b1)&&(IO_Write_Strobe==1'b1))
       we_o <=  1'b1 ;
    else if (ack_o==1'b1)
       we_o <=  1'b0 ;
    else
       we_o <=  we_o ;


always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    adr_o <= 32'h00000000;
  else 
    if (IO_Addr_Strobe==1'b1)
       adr_o <= IO_Address[31:0]  ;
    else
       adr_o <=  adr_o ;
   
always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    sel_o <= 4'b0000;
  else 
    if (IO_Addr_Strobe==1'b1)
       sel_o <=  IO_Byte_Enable  ;
    else
       sel_o <= sel_o ;

always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    IO_Ready <= 1'b0;
  else
    if ((iobus_mig_sel==1'b1)&&((IO_Read_Strobe==1'b1)||(IO_Write_Strobe==1'b1)))
      IO_Ready <= 1'b0;
    else if (ack_o==1'b1)      
      IO_Ready <=  1'b1 ;
    else if ( IO_Ready == 1'b1 )
      IO_Ready <= 1'b0;      
    else 
      IO_Ready <= IO_Ready ;

       

always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    dat_o <= 32'h00000000;
  else 
    if ((iobus_mig_sel==1'b1)&&(IO_Write_Strobe==1'b1))
       dat_o <= IO_Write_Data;  
    else
       dat_o <= dat_o ;
        
always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    IO_Read_Data <= 32'h00000000;
  else 
    if ((iobus_mig_sel==1'b1)&&(ack_o==1'b1))
      IO_Read_Data <= r_dat_o ;
    else
      IO_Read_Data <= IO_Read_Data ;
   
   
//Xe[g}V  
always @ (posedge Clk or negedge reset_n )
if (reset_n==1'b0)
  begin
    state <= p_nop;
    waite_cnt <= t_clear;
  end
else
  case (state)
    //-------------------------------------------------
    p_nop:begin
      if ((c3_p0_cmd_empty==1'b0)||(c3_p0_wr_full==1'b1))
        state <= p_nop; 
      else if ((cyc_o==1'b1)&&(we_o==1'b0)) 
        state <= p_rd_act;
      else if ((cyc_o==1'b1)&&(we_o==1'b1))
        state <= p_wr_act;
      else
        state <= p_nop;
      waite_cnt <= t_clear;
      end // 
    //-------------------------------------------------
    p_rd_act:begin
      if (waite_cnt==t_rd_act_end)
        begin
          state <= p_rd_wait;
          waite_cnt <= t_clear;
        end
      else
        begin
          state <= p_rd_act;
           waite_cnt <= waite_cnt + t_inc;
        end
      end // 
    //-------------------------------------------------
    p_rd_wait:begin
      //if ((c3_p0_rd_empty==1'b0)||(waite_cnt==t_rd_wait_end))
      //if (( c3_p0_rd_count>=7'd3)||(waite_cnt==t_rd_wait_end))
      //if (( c3_p0_rd_count>=7'd7)||(waite_cnt==t_rd_wait_end))
      if (( c3_p0_rd_count>=7'd1)||(waite_cnt==t_rd_wait_end))
        begin
          state <= p_rd1;
          waite_cnt <= t_clear;
        end
      else
        begin
          state <= p_rd_wait;
          waite_cnt <= waite_cnt + t_inc;
        end
      end // 
    //-------------------------------------------------
    p_rd1:begin
      if (waite_cnt==t_rd1_end)
        begin
          //state <= p_rd_i;
          state <= p_nop;
          waite_cnt <= t_clear;
        end
      else
        begin
          state <= p_rd1;
          waite_cnt <= waite_cnt + t_inc;
        end
      end // 
    //-------------------------------------------------
    p_wr_act:begin
      if (waite_cnt==t_wr_act_end)
        begin
          state <= p_wr1;
          waite_cnt <= t_clear;
        end
      else
        begin
          state <= p_wr_act;
           waite_cnt <= waite_cnt + t_inc;
        end
      end // 
    //-------------------------------------------------
    p_wr1:begin
      if (waite_cnt==t_wr1_end)
        begin
          //state <= p_wr_i;
          state <= p_nop;
          waite_cnt <= t_clear;
        end
      else
        begin
          state <= p_wr1;
          waite_cnt <= waite_cnt + t_inc;
        end
      end //
    default:
      begin
        state <= p_init;
        waite_cnt <= t_clear;
      end
  endcase


   
always @ (posedge Clk or negedge reset_n )
if (reset_n==1'b0)
  ref_cnt <= 16'h0000;
else
  if (state==p_ref)
    ref_cnt <= 16'h0000;
  else
    ref_cnt <= ref_cnt + 16'h0001;

//MIG signals

assign c3_p0_cmd_clk = Clk;   
assign c3_p0_wr_clk = Clk;   
assign c3_p0_rd_clk = Clk;
   
// sdram signels
   
always @ (posedge Clk or negedge reset_n )
if (reset_n==1'b0)
  begin
    c3_p0_cmd_en <= 1'b0;
    c3_p0_cmd_instr <= 3'b000;
    c3_p0_cmd_bl <=6'd0;
    c3_p0_cmd_byte_addr <= 30'h00000000;
    c3_p0_wr_en <= 1'b0;
    c3_p0_wr_mask <= 4'b0000;
    c3_p0_wr_data <= 32'h00000000;
    c3_p0_rd_en <= 1'b0;
  end
else
  if (waite_cnt==16'h0000)
  case (state)
    //-------------------------------------------------
    p_nop:
    begin
      c3_p0_cmd_en <= 1'b0;
      c3_p0_cmd_instr <= 3'b000;
      c3_p0_cmd_bl <=6'd0;
      c3_p0_cmd_byte_addr <= c3_p0_cmd_byte_addr;
      c3_p0_wr_en <= 1'b0;
      c3_p0_wr_mask <= 4'b0000;
      c3_p0_wr_data <= 32'h00000000;
      c3_p0_rd_en <= 1'b0;
    end
    //-------------------------------------------------
    p_rd_act:
    begin
      c3_p0_cmd_en <= 1'b1;
      c3_p0_cmd_instr <= 3'b001;
      c3_p0_cmd_bl <=6'd0;
      c3_p0_cmd_byte_addr <= {2'h0,adr_o};
      c3_p0_wr_en <= 1'b0;
      c3_p0_wr_mask <= 4'b0000;
      c3_p0_wr_data <= 32'h00000000;
      c3_p0_rd_en <= 1'b0;
    end
    //-------------------------------------------------
    p_rd_wait:
    begin
      c3_p0_cmd_en <= 1'b0;
      c3_p0_cmd_instr <= 3'b000;
      c3_p0_cmd_bl <=6'd0;
      c3_p0_cmd_byte_addr <= c3_p0_cmd_byte_addr;
      c3_p0_wr_en <= 1'b0;
      c3_p0_wr_mask <= 4'b0000;
      c3_p0_wr_data <= 32'h00000000;
      c3_p0_rd_en <= 1'b0;
    end // case: p_rd1
    //-------------------------------------------------
    p_rd1:
    begin
      c3_p0_cmd_en <= 1'b0;
      c3_p0_cmd_instr <= 3'b000;
      c3_p0_cmd_bl <=6'd0;
      c3_p0_cmd_byte_addr <= c3_p0_cmd_byte_addr;
      c3_p0_wr_en <= 1'b0;
      c3_p0_wr_mask <= 4'b0000;
      c3_p0_wr_data <= 32'h00000000;
      c3_p0_rd_en <= 1'b1;
    end // case: p_rd1
    //-------------------------------------------------
    p_wr_act:
    begin
      c3_p0_cmd_en <= 1'b0;
      c3_p0_cmd_instr <= 3'b000;
      c3_p0_cmd_bl <=6'd0;
      c3_p0_cmd_byte_addr <= {2'h0,adr_o};
      c3_p0_wr_en <= 1'b0;
      c3_p0_wr_mask <= ~sel_o;
      //c3_p0_wr_mask <= 4'b0000;
      c3_p0_wr_data <= 32'h00000000;
      c3_p0_rd_en <= 1'b0;
    end
    //-------------------------------------------------
    p_wr1:
    begin
      c3_p0_cmd_en <= 1'b0;
      c3_p0_cmd_instr <= 3'b000;
      c3_p0_cmd_bl <=6'd0;
      c3_p0_cmd_byte_addr <= c3_p0_cmd_byte_addr;
      //c3_p0_wr_en <= 1'b1; // 2nd byte enable
      c3_p0_wr_en <= 1'b0; // 2nd byte disenable
      c3_p0_wr_mask <= c3_p0_wr_mask;
      c3_p0_wr_data <= dat_o; //2nd byte
      c3_p0_rd_en <= 1'b0;
    end // case: p_wr
    //-------------------------------------------------
    default:
    begin
      c3_p0_cmd_en <= 1'b0;
      c3_p0_cmd_instr <= 3'b000;
      c3_p0_cmd_bl <=6'd0;
      c3_p0_cmd_byte_addr <= 30'h00000000;
      c3_p0_wr_en <= 1'b0;
      c3_p0_wr_mask <= 4'b0000;
      c3_p0_wr_data <= 32'h00000000;
      c3_p0_rd_en <= 1'b0;
    end
  endcase // case(state)
  else
    begin
      if  ((state==p_wr1)&&(waite_cnt==t_wr_cmd))
        begin
          c3_p0_cmd_en <= 1'b1;
          c3_p0_cmd_byte_addr <= {2'h0,adr_o};
        end
      else
        begin
          c3_p0_cmd_en <= 1'b0;
          c3_p0_cmd_byte_addr <= c3_p0_cmd_byte_addr;
          c3_p0_cmd_bl <=6'd0;
        end
      c3_p0_cmd_instr <= 3'b000;
      //c3_p0_wr_mask <= 4'b0000;
      c3_p0_wr_mask <= c3_p0_wr_mask ;
      c3_p0_cmd_bl <= c3_p0_cmd_bl;
      //if (((state==p_wr1)&&(waite_cnt<=t_wr_dqm_end))||    //3rd,4th byte
      //    ((state==p_wr_act)&&(waite_cnt==t_wr_dqm_start ))) //1st byte
      if ((state==p_wr_act)&&(waite_cnt==t_wr_dqm_start )) //1st word
        begin   
          c3_p0_wr_data <= dat_o;
          c3_p0_wr_en <= 1'b1;
        end
      else
        begin 
          c3_p0_wr_data <= 32'h00000000;
          c3_p0_wr_en <= 1'b0;
        end
      if ((state==p_rd1)&&(waite_cnt>=t_rd_dqm_start)&&(waite_cnt<=t_rd_dqm_end))
        begin   
          c3_p0_rd_en <= 1'b1;
        end
      else
        begin 
          c3_p0_rd_en <= 1'b0;
        end
    end

// user_if signels
   
always @ (posedge Clk or negedge reset_n )
if (reset_n==1'b0)
  begin
    ack_o <= 1'b0;
  end
else
  begin
    // u_wack
    if ((state==p_wr1)&&(waite_cnt==16'h0000))
      ack_o <= 1'b1;
    else if (c3_p0_rd_en==1'b1)
      ack_o <= 1'b1;
    else
      ack_o <= 1'b0;
  end
     
      
always @ (posedge Clk or negedge reset_n )
if (reset_n==1'b0)
   r_dat_o <= 32'h0000;
else
   r_dat_o <= c3_p0_rd_data;
   
endmodule








