`timescale 1ns / 1ps
//*****************************************************************************
// File Name            : mb_mcs_sys.v
//-----------------------------------------------------------------------------
// Function             : MicroBlaze MCS system 
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2013/02/26
// -.-- 2013/07/10
//*****************************************************************************

module mb_mcs_sys(  Clk, Reset, UART_Rx, UART_Tx,led,dip_sw,
 //DVI
 DVI_TX0_P,DVI_TX1_P,DVI_TX2_P,DVI_TXC_P,
 //LPDDR
 mcb3_dram_dq,   
 mcb3_dram_a,    
 mcb3_dram_ba,                   
 mcb3_dram_cke,  
 mcb3_dram_ras_n,
 mcb3_dram_cas_n,
 mcb3_dram_we_n, 
 mcb3_dram_dm,   
 mcb3_dram_udqs, 
 mcb3_rzq,       
 mcb3_dram_udm,  
 mcb3_dram_dqs,  
 mcb3_dram_ck,   
 mcb3_dram_ck_n,
 //
 mon
    );

  input  Clk;      //input clock 100MHz
  input  Reset;    //reset Hi=reset
  input  UART_Rx;  //UART rx 115200bps
  output UART_Tx; //UART tx 115200bps
  output [3 : 0] led;   //led Hi=on
  input  [3 : 0] dip_sw; //input dip_sw
  //dvi
  output        DVI_TX0_P;
  output        DVI_TX1_P;
  output        DVI_TX2_P;
  output        DVI_TXC_P;
  //lpddr 
  inout [15:0] mcb3_dram_dq;   
  output [12:0]              mcb3_dram_a;    
  output [1:0]               mcb3_dram_ba;   
  output                     mcb3_dram_cke;  
  output                     mcb3_dram_ras_n;
  output                     mcb3_dram_cas_n;
  output                     mcb3_dram_we_n; 
  output                     mcb3_dram_dm;   
  inout                      mcb3_dram_udqs; 
  inout                      mcb3_rzq;       
  output                     mcb3_dram_udm;  
  inout                      mcb3_dram_dqs;  
  output                     mcb3_dram_ck;   
  output                     mcb3_dram_ck_n;  
  //monitor     
  output mon;
  // 
  parameter p_reg_addr_low =  32'hC0000000;
  parameter p_reg_addr_hi  =  32'hC0000007;
  // 
  parameter p_vga_addr_low =  32'hC0000100;
  parameter p_vga_addr_hi  =  32'hC0000107;
  // 
  parameter p_bram_addr_low = 32'hC0001000;
  parameter p_bram_addr_hi  = 32'hC00017FF;
  // 
  parameter p_mig_addr_low = 32'hD0000000;
  parameter p_mig_addr_hi  = 32'hD3FFFFFF;
  //clk reset
  wire clk_100m_i;
  wire pclk_i;
  wire pclk_ix5;
  wire pclk_ix2;
  wire locked;
  wire resetb;
  wire mig_reset;
  wire mcs_reset;
  wire mcs_resetb;
   
  //IO Bus
  wire IO_Addr_Strobe;
  wire IO_Read_Strobe;
  wire IO_Write_Strobe;
  wire [31 : 0] IO_Read_Data;
  wire [31 : 0] IO_Read_Data_reg;
  wire [31 : 0] IO_Read_Data_vga;
  wire [31 : 0] IO_Read_Data_bram;
  wire [31 : 0] IO_Read_Data_mig;
  wire [31 : 0] IO_Address;
  wire [3 : 0]  IO_Byte_Enable;
  wire [31 : 0] IO_Write_Data;
  wire IO_Ready;
  wire IO_Ready_reg;
  wire IO_Ready_vga;
  wire IO_Ready_bram;
  wire IO_Ready_mig;
  wire iobus_reg_sel;
  wire iobus_vga_sel;
  wire iobus_bram_sel;   
  wire iobus_mig_sel;   
  //timmer
  wire FIT1_Interrupt;
  wire FIT1_Toggle;    
  wire PIT1_Interrupt; 
  wire PIT1_Toggle; 
  //GPO   
  wire [31 : 0] GPO1;
  wire [31 : 0] GPI1;
  //GPI
  wire GPI1_Interrupt;
  //Interrupt controller
  wire INTC_Interrupt;
  wire INTC_IRQ;
  //iobus_reg
  wire [31:0] data_in;
  wire [3:0]  data_in_en;
  wire [31:0] data_out;
  wire [3:0]  data_out_en;
  //user_module
  wire [7:0] chk_data; 
  //block ram port-B
  wire  [3:0]web;
  wire  [8 : 0] addrb;
  wire  [31 : 0] dinb;
  wire  [31 : 0] doutb;
  ///vga
   
  wire            hsync;               // horizontal sync
  wire            vsync;               // vertical sync
  wire   [ 7:0]  red, green, blue; // RGB color signals
  wire   da_en;
   
  //mig
  wire        c3_sys_clk; //200MHz lpddr
  wire        c3_p0_cmd_clk;
  wire        c3_p0_cmd_en;
  wire [2:0]  c3_p0_cmd_instr;
  wire [5:0]  c3_p0_cmd_bl;
  wire [29:0] c3_p0_cmd_byte_addr;
  wire         c3_p0_cmd_empty;
  wire         c3_p0_cmd_full;
  wire        c3_p0_wr_clk;
  wire        c3_p0_wr_en;
  wire [3:0]  c3_p0_wr_mask;
  wire [31:0] c3_p0_wr_data;
  wire         c3_p0_wr_full;
  wire         c3_p0_wr_empty;
  wire [6:0]   c3_p0_wr_count;
  wire         c3_p0_wr_underrun;
  wire         c3_p0_wr_error;
  wire        c3_p0_rd_clk;
  wire        c3_p0_rd_en;
  wire [31:0]  c3_p0_rd_data;
  wire         c3_p0_rd_full;
  wire         c3_p0_rd_empty;
  wire [6:0]   c3_p0_rd_count;
  wire         c3_p0_rd_overflow;
  wire        c3_p0_rd_error  ;
  wire        c3_p2_cmd_clk;
  wire        c3_p2_cmd_en ;
  wire [2:0]  c3_p2_cmd_instr ;
  wire [5:0]  c3_p2_cmd_bl ;
  wire [29:0] c3_p2_cmd_byte_addr ;
  wire         c3_p2_cmd_empty ;
  wire        c3_p2_cmd_full; 
  wire        c3_p2_wr_clk  ;
  wire        c3_p2_wr_en  ;
  wire [3:0]  c3_p2_wr_mask  ;
  wire [31:0] c3_p2_wr_data  ;
  wire         c3_p2_wr_full = 0;
  wire         c3_p2_wr_empty = 0;
  wire [6:0]   c3_p2_wr_count = 0 ;
  wire         c3_p2_wr_underrun = 0;
  wire         c3_p2_wr_error = 0;
  wire        c3_p2_rd_clk ;
  wire        c3_p2_rd_en ;
  wire [31:0]  c3_p2_rd_data;
  wire         c3_p2_rd_full;
  wire         c3_p2_rd_empty;
  wire [6:0]   c3_p2_rd_count;
  wire         c3_p2_rd_overflow;
  wire        c3_p2_rd_error  ;
  wire        c3_calib_done  ;

clk_gen clk_gen
   (// Clock in ports
   .CLK_IN1(Clk),      // IN  100MHz
   // Clock out ports
   .CLK_OUT1(clk_100m_i),   // OUT 100MHz
   .CLK_OUT2(pclk_i),     // OUT 40MHz
   .CLK_OUT3(pclk_ix5),       // OUT 200MHz
   .CLK_OUT4(pclk_x2),      // OUT  80MHz
   .CLK_OUT5(c3_sys_clk),   // OUT 200MHz lpddr clk
   // Status and control signals
   .RESET(reset ),          // IN
   .LOCKED(locked)          // OUT 
    );

//
   
  assign resetb = ~Reset;
  assign mig_reset =((resetb==1'b0)||(locked==1'b0))?1'b1:1'b0;
  assign mcs_reset =((resetb==1'b0)||(locked==1'b0)||(c3_calib_done==1'b0))?1'b1:1'b0;
  assign mcs_resetb = ~mcs_reset ;
   
  //LED
  assign led = {data_out[1:0],GPO1[1:0]};
   //Mڑ
  assign GPI1 = {28'h0000000,dip_sw};
  assign data_in = data_out;
  assign data_in_en = data_out_en; 
  assign chk_data = data_out[7:0];  
  //assign INTC_Interrupt = 1'b0;  

  //ӉHIM
  assign iobus_reg_sel  = ((IO_Address>=p_reg_addr_low)&&(IO_Address<=p_reg_addr_hi))? 1'b1:1'b0;
  assign iobus_vga_sel  = ((IO_Address>=p_vga_addr_low)&&(IO_Address<=p_vga_addr_hi))? 1'b1:1'b0;
  assign iobus_bram_sel = ((IO_Address>=p_bram_addr_low)&&(IO_Address<=p_bram_addr_hi))? 1'b1:1'b0;  
  assign iobus_mig_sel = ((IO_Address>=p_mig_addr_low)&&(IO_Address<=p_mig_addr_hi))? 1'b1:1'b0;   
  //Ǐof[^̑I       
  assign IO_Read_Data = (iobus_reg_sel==1'b1)?  IO_Read_Data_reg:
                        (iobus_vga_sel==1'b1)?  IO_Read_Data_vga:
                        (iobus_bram_sel==1'b1)? IO_Read_Data_bram:
                        (iobus_mig_sel==1'b1)?  IO_Read_Data_mig:
                                                32'h00000000;
  //ReadyM̑I  
  assign IO_Ready =    (iobus_reg_sel==1'b1)?  IO_Ready_reg:
                       (iobus_vga_sel==1'b1)?  IO_Ready_vga:
                       (iobus_bram_sel==1'b1)? IO_Ready_bram:
                       (iobus_mig_sel==1'b1)?  IO_Ready_mig:
                                               1'b0;

//Microblaze MCS̃CX^X
mb_mcs mcs_0 (
  .Clk(clk_100m_i), // input Clk
  .Reset(mcs_reset), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready), // input IO_Ready
  .UART_Rx(UART_Rx), // input UART_Rx
  .UART_Tx(UART_Tx), // output UART_Tx
  .FIT1_Interrupt(FIT1_Interrupt), // output FIT1_Interrupt
  .FIT1_Toggle(FIT1_Toggle), // output FIT1_Toggle
  .PIT1_Interrupt(PIT1_Interrupt), // output PIT1_Interrupt
  .PIT1_Toggle(PIT1_Toggle), // output PIT1_Toggle
  .GPO1(GPO1), // output [31 : 0] GPO1
  .GPI1(GPI1), // input [31 : 0] GPI1
  .GPI1_Interrupt(GPI1_Interrupt), // output GPI1_Interrupt
  .INTC_Interrupt(INTC_Interrupt), // input [0 : 0] INTC_Interrupt
  .INTC_IRQ(INTC_IRQ) // output INTC_IRQ
); 
 
//ݒ背WX^̃CX^X
iobus_reg
#(p_reg_addr_low, p_reg_addr_hi)
iobus_reg_0 (
  .Clk(clk_100m_i), // input Clk
  .Reset(mcs_reset), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data_reg), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready_reg), // input IO_Ready
  //input 
  .data_in(data_in), // input data
  .data_in_en(data_in_en), // input data enable
  //output 
  .data_out(data_out), // output data
  .data_out_en(data_out_en) // output data enable
);

//ubNRAMC^[tF[X̃CX^X
iobus_bram
 #(p_bram_addr_low, p_bram_addr_hi)
iobus_bram_0 (
  .Clk(clk_100m_i), // input Clk
  .Reset(mcs_reset), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data_bram), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready_bram), // input IO_Ready
  //BRAM Port-B                  
  .clkb(clk_100m_i), // input clkb
  .web(web), // input [3:0]web
  .addrb(addrb), // input [8 : 0] addrb
  .dinb(dinb), // input [31 : 0] dinb
  .doutb(doutb) // output [31 : 0] doutb
);
   
user_module user_module_0 (
  .Clk(clk_100m_i), // input Clk
  .Reset(mcs_reset), // input Reset
  .chk_data(chk_data),
  //BRAM Port-B                  
  .clk(clkb), // input clkb
  .we(web), // input[3:0]web
  .addr(addrb), // input [8 : 0] addrb
  .dout(dinb) // output [31 : 0] doutb
);


 //IOBUS <-> MIG
iobus_2_mig 
iobus_2_mig (
  //io_bus
  .Clk(clk_100m_i), // input Clk
  .Reset(mcs_reset), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data_mig), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready_mig), // input IO_Ready
  .iobus_mig_sel( iobus_mig_sel),
  //mig if
  //.clk(      c3_clk0),           
  .reset_n(  mcs_resetb),
  .c3_p0_cmd_clk                          (c3_p0_cmd_clk),
  .c3_p0_cmd_en                           (c3_p0_cmd_en),
  .c3_p0_cmd_instr                        (c3_p0_cmd_instr),
  .c3_p0_cmd_bl                           (c3_p0_cmd_bl),
  .c3_p0_cmd_byte_addr                    (c3_p0_cmd_byte_addr),
  .c3_p0_cmd_empty                        (c3_p0_cmd_empty),
  .c3_p0_cmd_full                         (c3_p0_cmd_full),
  .c3_p0_wr_clk                           (c3_p0_wr_clk),
  .c3_p0_wr_en                            (c3_p0_wr_en),
  .c3_p0_wr_mask                          (c3_p0_wr_mask),
  .c3_p0_wr_data                          (c3_p0_wr_data),
  .c3_p0_wr_full                          (c3_p0_wr_full),
  .c3_p0_wr_empty                         (c3_p0_wr_empty),
  .c3_p0_wr_count                         (c3_p0_wr_count),
  .c3_p0_wr_underrun                      (c3_p0_wr_underrun),
  .c3_p0_wr_error                         (c3_p0_wr_error),
  .c3_p0_rd_clk                           (c3_p0_rd_clk),
  .c3_p0_rd_en                            (c3_p0_rd_en),
  .c3_p0_rd_data                          (c3_p0_rd_data),
  .c3_p0_rd_full                          (c3_p0_rd_full),
  .c3_p0_rd_empty                         (c3_p0_rd_empty),
  .c3_p0_rd_count                         (c3_p0_rd_count),
  .c3_p0_rd_overflow                      (c3_p0_rd_overflow),
  .c3_p0_rd_error                         (c3_p0_rd_error),
  .c3_calib_done          (c3_calib_done)
                         );
   
   
//VGA_CTRL

   
 svga_ctrl 
 #(p_vga_addr_low, p_vga_addr_hi)
 svga_ctrl (
  .Clk(clk_100m_i), // input Clk
  .Reset(mcs_reset), // input Reset
  .IO_Addr_Strobe(IO_Addr_Strobe), // output IO_Addr_Strobe
  .IO_Read_Strobe(IO_Read_Strobe), // output IO_Read_Strobe
  .IO_Write_Strobe(IO_Write_Strobe), // output IO_Write_Strobe
  .IO_Address(IO_Address), // output [31 : 0] IO_Address
  .IO_Byte_Enable(IO_Byte_Enable), // output [3 : 0] IO_Byte_Enable
  .IO_Write_Data(IO_Write_Data), // output [31 : 0] IO_Write_Data
  .IO_Read_Data(IO_Read_Data_vga), // input [31 : 0] IO_Read_Data
  .IO_Ready(IO_Ready_vga), // input IO_Ready
  .pclk(pclk_i), 
  .hsync(hsync), 
  .vsync(vsync), 
  .de(da_en), 
  .red(red), 
  .green(green), 
  .blue(blue), 
  //.c3_clk0                (clk_i),
  .c3_p0_cmd_clk(c3_p2_cmd_clk), 
  .c3_p0_cmd_en(c3_p2_cmd_en), 
  .c3_p0_cmd_instr(c3_p2_cmd_instr), 
  .c3_p0_cmd_bl(c3_p2_cmd_bl), 
  .c3_p0_cmd_byte_addr(c3_p2_cmd_byte_addr), 
  .c3_p0_cmd_empty(c3_p2_cmd_empty), 
  .c3_p0_cmd_full(c3_p2_cmd_full), 
  .c3_p0_wr_clk(c3_p2_wr_clk), 
  .c3_p0_wr_en(c3_p2_wr_en), 
  .c3_p0_wr_mask(c3_p2_wr_mask), 
  .c3_p0_wr_data(c3_p2_wr_data), 
  .c3_p0_wr_full(c3_p2_wr_full), 
  .c3_p0_wr_empty(c3_p2_wr_empty), 
  .c3_p0_wr_count(c3_p2_wr_count), 
  .c3_p0_wr_underrun(c3_p2_wr_underrun), 
  .c3_p0_wr_error(c3_p2_wr_error), 
  .c3_p0_rd_clk(c3_p2_rd_clk), 
  .c3_p0_rd_en(c3_p2_rd_en), 
  .c3_p0_rd_data(c3_p2_rd_data), 
  .c3_p0_rd_full(c3_p2_rd_full), 
  .c3_p0_rd_empty(c3_p2_rd_empty), 
  .c3_p0_rd_count(c3_p2_rd_count), 
  .c3_p0_rd_overflow(c3_p2_rd_overflow), 
  .c3_p0_rd_error(c3_p2_rd_error), 
  .c3_calib_done(c3_calib_done),
  .mon(mig_2_v_mon)
        );

   
//DVI Output
 xil_dvi #(
        .differential(0)                                // select diff=1 , single_end=0
) dvi_out (
        .DVI_TX0_N(),                                   // out   [TX] TX[0]-N (CML)
        .DVI_TX0_P(DVI_TX0_P),              // out   [TX] TX[0]-P (CML)
        .DVI_TX1_N(),                                   // out   [TX] TX[1]-N (CML)
        .DVI_TX1_P(DVI_TX1_P),              // out   [TX] TX[1]-P (CML)
        .DVI_TX2_N(),                                   // out   [TX] TX[2]-N (CML)
        .DVI_TX2_P(DVI_TX2_P),              // out   [TX] TX[2]-P (CML)
        .DVI_TXC_N(),                                   // out   [TX] TX_CLK-N (CML)
        .DVI_TXC_P(DVI_TXC_P),             // out   [TX] TX_CLK-P (CML)

        .TX_RED(red),                   // in    [TX] [7:0] red
        .TX_GRN(green),                   // in    [TX] [7:0] green
        .TX_BLU(blue),                   // in    [TX] [7:0] blue
        .TX_HS(hsync),                                  // in    [TX] hsync
        .TX_VS(vsync),                                  // in    [TX] vsync
        .TX_C0(1'b0),                                   // in    [TX] c0
        .TX_C1(1'b0),                                   // in    [TX] c1
        .TX_C2(1'b0),                                   // in    [TX] c2
        .TX_C3(1'b0),                                   // in    [TX] c3
        .TX_DE(da_en),                                  // in    [TX] de

        .CLK(pclk_i),                                    // in    [DVI] clk
        .CLKx5(!pclk_ix5),                               // in    [DVI] clk x5 : dvi ddr
        .RESET_N(mcs_resetb)                                 // in    [DVI] #reset
);

      
  //mig
   parameter DEBUG_EN              = 0;
   parameter C3_MEMCLK_PERIOD      = 5000;
   parameter C3_RST_ACT_LOW        = 0;
   parameter C3_INPUT_CLK_TYPE     = "SINGLE_ENDED";
   parameter C3_NUM_DQ_PINS        = 16;
   parameter C3_MEM_ADDR_WIDTH     = 13;
   parameter C3_MEM_BANKADDR_WIDTH = 2;   
   parameter C3_MEM_ADDR_ORDER     = "ROW_BANK_COLUMN"; 
   parameter C3_P0_MASK_SIZE       = 4;
   parameter C3_P0_DATA_PORT_SIZE  = 32;  
   parameter C3_P1_MASK_SIZE       = 4;
   parameter C3_P1_DATA_PORT_SIZE  = 32;
   parameter C3_MEM_BURST_LEN     = 4;
   parameter C3_MEM_NUM_COL_BITS   = 10;
   parameter C3_CALIB_SOFT_IP      = "TRUE";  
   parameter C3_SIMULATION      = "TRUE";
   parameter C3_HW_TESTING      = "FALSE";
   parameter C3_SMALL_DEVICE    = "FALSE";
   
MIG #(
.C3_P0_MASK_SIZE       (C3_P0_MASK_SIZE      ),
.C3_P0_DATA_PORT_SIZE  (C3_P0_DATA_PORT_SIZE ),
.C3_P1_MASK_SIZE       (C3_P1_MASK_SIZE      ),
.C3_P1_DATA_PORT_SIZE  (C3_P1_DATA_PORT_SIZE ),
.C3_MEMCLK_PERIOD      (C3_MEMCLK_PERIOD),
.C3_RST_ACT_LOW        (C3_RST_ACT_LOW),
.C3_INPUT_CLK_TYPE     (C3_INPUT_CLK_TYPE),
.DEBUG_EN              (DEBUG_EN),
.C3_MEM_ADDR_ORDER     (C3_MEM_ADDR_ORDER    ),
.C3_NUM_DQ_PINS        (C3_NUM_DQ_PINS       ),
.C3_MEM_ADDR_WIDTH     (C3_MEM_ADDR_WIDTH    ),
.C3_MEM_BANKADDR_WIDTH (C3_MEM_BANKADDR_WIDTH),
.C3_SIMULATION         (C3_SIMULATION),
.C3_CALIB_SOFT_IP      (C3_CALIB_SOFT_IP )
)
MIG  (
  .c3_sys_clk             (c3_sys_clk),
  .c3_sys_rst_i           (mig_reset),                        
  .mcb3_dram_dq           (mcb3_dram_dq),  
  .mcb3_dram_a            (mcb3_dram_a),  
  .mcb3_dram_ba           (mcb3_dram_ba),
  .mcb3_dram_ras_n        (mcb3_dram_ras_n),                        
  .mcb3_dram_cas_n        (mcb3_dram_cas_n),                        
  .mcb3_dram_we_n         (mcb3_dram_we_n),                          
  .mcb3_dram_cke          (mcb3_dram_cke),                          
  .mcb3_dram_ck           (mcb3_dram_ck),                          
  .mcb3_dram_ck_n         (mcb3_dram_ck_n),       
  .mcb3_dram_dqs          (mcb3_dram_dqs),
  .mcb3_dram_udqs         (mcb3_dram_udqs),    // for X16 parts
  .mcb3_dram_udm          (mcb3_dram_udm),     // for X16 parts
  .mcb3_dram_dm           (mcb3_dram_dm),
  .c3_clk0                (c3_clk0),
  .c3_rst0                (c3_rst0),
  .c3_calib_done          (c3_calib_done),
  //.mcb3_rzq               (rzq3),
  .mcb3_rzq               (mcb3_rzq),
  .c3_p0_cmd_clk                          (c3_p0_cmd_clk),
  .c3_p0_cmd_en                           (c3_p0_cmd_en),
  .c3_p0_cmd_instr                        (c3_p0_cmd_instr),
  .c3_p0_cmd_bl                           (c3_p0_cmd_bl),
  .c3_p0_cmd_byte_addr                    (c3_p0_cmd_byte_addr),
  .c3_p0_cmd_empty                        (c3_p0_cmd_empty),
  .c3_p0_cmd_full                         (c3_p0_cmd_full),
  .c3_p0_wr_clk                           (c3_p0_wr_clk),
  .c3_p0_wr_en                            (c3_p0_wr_en),
  .c3_p0_wr_mask                          (c3_p0_wr_mask),
  .c3_p0_wr_data                          (c3_p0_wr_data),
  .c3_p0_wr_full                          (c3_p0_wr_full),
  .c3_p0_wr_empty                         (c3_p0_wr_empty),
  .c3_p0_wr_count                         (c3_p0_wr_count),
  .c3_p0_wr_underrun                      (c3_p0_wr_underrun),
  .c3_p0_wr_error                         (c3_p0_wr_error),
  .c3_p0_rd_clk                           (c3_p0_rd_clk),
  .c3_p0_rd_en                            (c3_p0_rd_en),
  .c3_p0_rd_data                          (c3_p0_rd_data),
  .c3_p0_rd_full                          (c3_p0_rd_full),
  .c3_p0_rd_empty                         (c3_p0_rd_empty),
  .c3_p0_rd_count                         (c3_p0_rd_count),
  .c3_p0_rd_overflow                      (c3_p0_rd_overflow),
  .c3_p0_rd_error                         (c3_p0_rd_error),
  .c3_p2_cmd_clk                          (c3_p2_cmd_clk),
  .c3_p2_cmd_en                           (c3_p2_cmd_en),
  .c3_p2_cmd_instr                        (c3_p2_cmd_instr),
  .c3_p2_cmd_bl                           (c3_p2_cmd_bl),
  .c3_p2_cmd_byte_addr                    (c3_p2_cmd_byte_addr),
  .c3_p2_cmd_empty                        (c3_p2_cmd_empty),
  .c3_p2_cmd_full                         (c3_p2_cmd_full),
  .c3_p2_rd_clk                           (c3_p2_rd_clk),
  .c3_p2_rd_en                            (c3_p2_rd_en),
  .c3_p2_rd_data                          (c3_p2_rd_data),
  .c3_p2_rd_full                          (c3_p2_rd_full),
  .c3_p2_rd_empty                         (c3_p2_rd_empty),
  .c3_p2_rd_count                         (c3_p2_rd_count),
  .c3_p2_rd_overflow                      (c3_p2_rd_overflow),
  .c3_p2_rd_error                         (c3_p2_rd_error)
);

   assign    mon = locked ^ c3_p2_cmd_en ^ c3_p2_rd_empty;
   
   
endmodule
