//*****************************************************************************
// File Name            : vga_ctrl.v
//-----------------------------------------------------------------------------
// Function             : vga_ctrl 
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo@himwari_co 
//-----------------------------------------------------------------------------
// History
// -.-- 2013/09/11
//*****************************************************************************
module svga_ctrl (
//iobus                 
  Clk, Reset, IO_Ready, IO_Addr_Strobe, IO_Read_Strobe, IO_Write_Strobe,  IO_Read_Data, IO_Address, IO_Byte_Enable, IO_Write_Data,
  //video  
  pclk,
  hsync,          // hsync data
  vsync,          // vsync data
  de,             // data enable  
  red,      // pixel data out
  green,    // pixel data out
  blue,   // pixel data out
  //mmig
  c3_p0_cmd_clk,
  c3_p0_cmd_en,
  c3_p0_cmd_instr,
  c3_p0_cmd_bl,
  c3_p0_cmd_byte_addr,
  c3_p0_cmd_empty,
  c3_p0_cmd_full,
  c3_p0_wr_clk,
  c3_p0_wr_en,
  c3_p0_wr_mask,
  c3_p0_wr_data,
  c3_p0_wr_full,
  c3_p0_wr_empty,
  c3_p0_wr_count,
  c3_p0_wr_underrun,
  c3_p0_wr_error,
  c3_p0_rd_clk,
  c3_p0_rd_en,
  c3_p0_rd_data,
  c3_p0_rd_full,
  c3_p0_rd_empty,
  c3_p0_rd_count,
  c3_p0_rd_overflow,
  c3_p0_rd_error,
  c3_calib_done ,
  mon
);
//io_bus
  input Clk;
  input Reset;
  input IO_Addr_Strobe;
  input IO_Read_Strobe;
  input IO_Write_Strobe;
  input [31 : 0] IO_Address;
  input [3 : 0] IO_Byte_Enable;
  input [31 : 0] IO_Write_Data;
  output [31 : 0] IO_Read_Data;
  output IO_Ready;
  //
  //video
  input         pclk;
  output  hsync;          // hsync data
  output  vsync;          // vsync data
  output  de;             // data enable
  output  [7:0] red;      // pixel data in
  output  [7:0] green;    // pixel data in
  output  [7:0] blue;    // pixel data in
  //MIG
  output        c3_p0_cmd_clk;
  output        c3_p0_cmd_en;
  output [2:0]  c3_p0_cmd_instr;
  output [5:0]  c3_p0_cmd_bl;
  output [29:0] c3_p0_cmd_byte_addr;
  input         c3_p0_cmd_empty;
  input         c3_p0_cmd_full;
  output        c3_p0_wr_clk;
  output        c3_p0_wr_en;
  output [3:0]  c3_p0_wr_mask;
  output [31:0] c3_p0_wr_data;
  input         c3_p0_wr_full;
  input         c3_p0_wr_empty;
  input [6:0]   c3_p0_wr_count;
  input         c3_p0_wr_underrun;
  input         c3_p0_wr_error;
  output        c3_p0_rd_clk;
  output        c3_p0_rd_en;
  input [31:0]  c3_p0_rd_data;
  input         c3_p0_rd_full;
  input         c3_p0_rd_empty;
  input [6:0]   c3_p0_rd_count;
  input         c3_p0_rd_overflow;
  input         c3_p0_rd_error; 
  input         c3_calib_done ;
  //
   output       mon;
  //
  parameter p_vga_addr_low = 32'hC0000100;
  parameter p_vga_addr_hi  = 32'hC0000107;
  //
  parameter p_mode_offset = 0;
  parameter p_rgb_data_offset = 1;
  // 
  parameter  p_nop  = 4'd0;   
  parameter  p_rd_act  = 4'd1;
  parameter  p_rd_wait  = 4'd2;
  parameter  p_rd1  = 4'd3;
  parameter  t_clear = 16'd0;
  parameter  t_inc   = 16'd1;
  parameter  t_rd_act_end = 16'd0;
  parameter  t_rd_wait_end = 16'd1000;
  //parameter  t_wr1_end = 16'd9; //bl=8  
  parameter  t_rd1_end = 16'd36; //bl=32  
  parameter  t_rd_start = 16'd4;   //
  parameter  t_rd_end = 16'd35;   //
  parameter  t_fifo_wr_start = 16'd4;   //
  parameter  t_fifo_wr_end = 16'd35;   //
   
  parameter  p_start_de  = 255;
   
  wire         reset_n;
  reg [3:0]   state;
  reg [15:0]  waite_cnt;
  reg   c3_p0_cmd_en;
  reg [2:0]     c3_p0_cmd_instr;
  reg [5:0]     c3_p0_cmd_bl;
  reg [29:0]    c3_p0_cmd_byte_addr;
  reg   c3_p0_wr_en;
  reg [3:0]     c3_p0_wr_mask;
  reg [31:0]    c3_p0_wr_data;
  reg   c3_p0_rd_en;
                
//
  wire rst ;
  reg vsync;
  reg hsync;
  reg de  ;
  //
  reg vsync_d1;
  reg vsync_d2;
  reg vsync_d3;
  reg hsync_d1;
  reg hsync_d2;
  reg hsync_d3;
  reg de_d1;
  reg de_d2;
  reg de_d3;
   
  wire [23:0] fifo_dout;
  wire [9:0]fifo_data_counnt;
  reg  fifo_rd_en;
  reg  fifo_wr_en;
  wire fifo_full;
  wire fifo_empty;
  reg [9:0]v_cnt;
  reg [10:0]h_cnt;
  reg [9:0]v_adr;
  reg [9:0]h_adr;
  wire [25:0] r_adr;
   
reg [31:0] IO_Read_Data;
reg IO_Ready;
//      
reg [1:0]mode;
reg  [23:0]  rgb_data;     
reg  [2:0]  c_cnt;       
reg  [7:0]  dat_a_r;      
reg  [7:0]  dat_a_g;      
reg  [7:0]  dat_a_b;      
     
wire hit_addr;
   
  assign rst = Reset ;
  assign reset_n =  ~Reset ;

//AhX̊mF   
assign hit_addr = ((IO_Addr_Strobe==1'b1)&&(IO_Address>=p_vga_addr_low)&&( IO_Address<=p_vga_addr_hi))?1'b1:1'b0;

//݃f[^̕ێ
always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    begin
      mode <= 2'b0;
      rgb_data <= 24'h808080;
    end
  else 
    if ((hit_addr==1'b1)&&(IO_Write_Strobe==1'b1))
      begin
        if(IO_Address[2]==p_mode_offset)
          if(IO_Byte_Enable[0]==1'b1) //bit7_0
            mode <= IO_Write_Data[1:0] ;
          else
            mode <= mode;
        if(IO_Address[2]==p_rgb_data_offset)
          begin
            if(IO_Byte_Enable[0]==1'b1) //bit7_0
              rgb_data[7:0] <= IO_Write_Data[7:0] ;
            else
              rgb_data[7:0] <=  rgb_data[7:0] ;
            if(IO_Byte_Enable[1]==1'b1) //bit15_8
              rgb_data[15:8] <= IO_Write_Data[15:8] ;
            else
              rgb_data[15:8] <=  rgb_data[15:8] ;
            if(IO_Byte_Enable[2]==1'b1) //bit23-16
              rgb_data[23:16] <= IO_Write_Data[23:16] ;
            else
              rgb_data[23:16] <=  rgb_data[23:16] ;
          end 
         else
          rgb_data <= rgb_data;
      end // if ((hit_addr==1'b1)&&(IO_Write_Strobe==1'b1))
    else
      begin
        mode <= mode;
        rgb_data <= rgb_data;
      end

   
//Readyʒm      
always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    IO_Ready <= 1'b1;
  else
    if  ((hit_addr==1'b1)&&((IO_Read_Strobe==1'b1)||(IO_Write_Strobe==1'b1)))
      IO_Ready <= 1'b1;
    else 
      IO_Ready <=  1'b0 ;

//Ǐof[^    
always @ (posedge Clk or posedge Reset )
  if (Reset==1'b1)
    IO_Read_Data <= 32'h00000000;
  else 
    if ((hit_addr==1'b1)&&(IO_Read_Strobe==1'b1))
      if (IO_Address[2]==p_mode_offset)
        IO_Read_Data <= {30'h00000000,mode};
      else if (IO_Address[2]==p_rgb_data_offset)
        IO_Read_Data <= {8'h00,rgb_data};
      else
        IO_Read_Data <= 32'h00000000;
    else
      IO_Read_Data <= IO_Read_Data ;

//vga timming
   
   
  always @ (posedge pclk or negedge reset_n )
  if (reset_n==1'b0)
    v_cnt <= 10'd0;
  else
    if (h_cnt==11'd1055)
      if (v_cnt==10'd627)
        v_cnt <= 10'd0;
      else
        v_cnt <= v_cnt + 10'd1;
     else
       v_cnt <= v_cnt;
   
  always @ (posedge pclk or negedge reset_n )
  if (reset_n==1'b0)
    h_cnt <= 11'd0;
  else
    if ( h_cnt==11'd1055)
       h_cnt <= 11'd0;
    else
       h_cnt <= h_cnt + 11'd1;


   
  always @ (posedge pclk or negedge reset_n )
  if (reset_n==1'b0)
    vsync <= 1'b1;
  else
    if ((v_cnt==10'd1)&& (h_cnt==11'd39))
      vsync <= 1'd0;
    else if ((v_cnt==10'd5)&& (h_cnt==11'd39))
      vsync <= 1'd1;
   else
      vsync <=  vsync ;
   
  always @ (posedge pclk or negedge reset_n )
  if (reset_n==1'b0)
    hsync <= 1'd1;
  else
    if (h_cnt==11'd39)
      hsync <= 1'd0;
    else if  (h_cnt==11'd167)
      hsync <= 1'd1;
    else
      hsync <= hsync;
   
  always @ (posedge pclk or negedge reset_n )
  if (reset_n==1'b0)
    de <= 1'b0;
  else
    if ((v_cnt>=10'd28)&&(v_cnt<=10'd627))
      if (h_cnt==p_start_de)
        de <= 10'd1;
      else if  (h_cnt==p_start_de + 800)
        de <= 10'd0;
      else
        de <= de;
      
  always @ (posedge pclk or negedge reset_n )
  if (reset_n==1'b0)
    fifo_rd_en <= 1'b0;
  else
    if ((v_cnt>=10'd28)&&(v_cnt<=10'd627))
      if (h_cnt==p_start_de -1)
        fifo_rd_en <= 1'b1;
      //else if  (h_cnt==11'd1053)
      else if  (h_cnt==p_start_de + 800)
        fifo_rd_en <= 1'b0;
      else
        fifo_rd_en <=  fifo_rd_en;

fifo_24b_1024w fifo_24b_1024w (
  .wr_clk(Clk), // input clk
  .rd_clk(pclk), // input clk
  .rst(rst), // input rst
  //.din(c3_p0_rd_data[25:0]), // input [25 : 0] din
  .din(c3_p0_rd_data[23:0]), // input [25 : 0] din
  .wr_en(fifo_wr_en), // input wr_en
  .rd_en(fifo_rd_en), // input rd_en
  .dout(fifo_dout), // output [25 : 0] dout
  .full(fifo_full), // output full
  .empty(fifo_empty) // output empty
  //.data_count(fifo_data_count) // output [8 : 0] data_count
);


 assign  red   = (mode==2'b01)? fifo_dout[23:16]:
                 (mode==2'b10)? dat_a_r[7:0]:
                 rgb_data[23:16];
 assign  green = (mode==2'b01)? fifo_dout[15:8]:
                 (mode==2'b10)? dat_a_g[7:0]:
                 rgb_data[15:8];
 assign  blue  = (mode==2'b01)? fifo_dout[7:0]:
                 (mode==2'b10)? dat_a_b[7:0]:
                 rgb_data[7:0];
   

always @ (posedge Clk or negedge reset_n )
  if (reset_n==1'b0)
    begin
       hsync_d1 <= 1'b0;
       hsync_d2 <= 1'b0;
       hsync_d3 <= 1'b0;
       vsync_d1 <= 1'b0;
       vsync_d2 <= 1'b0;
       vsync_d3 <= 1'b0;
       de_d1 <= 1'b0;
       de_d2 <= 1'b0;
       de_d3 <= 1'b0;
    end // if (reset_n==1'b0)
  else
    begin
       hsync_d1 <= hsync;
       hsync_d2 <= hsync_d1;
       hsync_d3 <= hsync_d2;
       vsync_d1 <= vsync;
       vsync_d2 <= vsync_d1;
       vsync_d3 <= vsync_d2;
       de_d1 <= de;
       de_d2 <= de_d1;
       de_d3 <= de_d2;
    end
   
always @ (posedge Clk or negedge reset_n )
  if (reset_n==1'b0)
    v_adr <= 10'd0;
  else
    if ((vsync_d2==1'b1)&&(vsync_d3==1'b0))
      v_adr <= 10'd0;
    else if  ((de_d2==1'b0)&&(de_d3==1'b1))
      v_adr <=  v_adr + 10'd1;
    else
      v_adr <=  v_adr ;
      

always @ (posedge Clk or negedge reset_n )
  if (reset_n==1'b0)
    h_adr <= 10'd0;
  else
    if ((de_d2==1'b0)&&(de_d3==1'b1))
      h_adr <= 10'd0;
    else
      if ((state==p_rd_act)&&(waite_cnt==t_rd_act_end))
          h_adr <= h_adr + 10'd32;
      else
        h_adr <= h_adr;

   assign  r_adr = (v_adr * 3200) + {h_adr,2'b00};
   
   
assign w_adr =   v_cnt*3200 + h_adr ;

assign rd_start = ((v_cnt>=10'd28)&&(v_cnt<=10'd627)&&(hsync_d2==1'b0)&&(hsync_d3==1'b1))?1'b1:1'b0 ;
    
//Xe[g}V  
always @ (posedge Clk or negedge reset_n )
if (reset_n==1'b0)
  begin
    state <= p_nop;
    waite_cnt <= t_clear;
  end
else
  case (state)
    //-------------------------------------------------
    p_nop:begin
      if (c3_p0_cmd_empty==1'b0)
        state <= p_nop; 
      else if ((rd_start==1'b1)||((h_adr>10'd0)&&(h_adr<10'd800)))
        state <= p_rd_act;
      else
        state <= p_nop;
      waite_cnt <= t_clear;
      end //
    
    p_rd_act:begin
      if (waite_cnt==t_rd_act_end)
        begin
          state <= p_rd_wait;
          waite_cnt <= t_clear;
        end
      else
        begin
          state <= p_rd_act;
           waite_cnt <= waite_cnt + t_inc;
        end
      end // 
    //-------------------------------------------------
    p_rd_wait:begin
      //if ((c3_p0_rd_empty==1'b0)||(waite_cnt==t_rd_wait_end))
      //if (( c3_p0_rd_count>=7'd3)||(waite_cnt==t_rd_wait_end))
      if (( c3_p0_rd_count>=7'd7)||(waite_cnt==t_rd_wait_end))
        begin
          state <= p_rd1;
          waite_cnt <= t_clear;
        end
      else
        begin
          state <= p_rd_wait;
          waite_cnt <= waite_cnt + t_inc;
        end
      end // 
    //-------------------------------------------------
    p_rd1:begin
      //if (waite_cnt>=t_rd1_end)
      if ((waite_cnt>=t_rd1_end)&&(c3_p0_rd_count==7'd0))
        begin
          //state <= p_rd_i;
          state <= p_nop;
          waite_cnt <= t_clear;
        end
      else
        begin
          state <= p_rd1;
          waite_cnt <= waite_cnt + t_inc;
        end
      end // 
    //-------------------------------------------------
    default:
      begin
        state <= p_nop;
        waite_cnt <= t_clear;
      end
  endcase


   

//MIG signals

assign c3_p0_cmd_clk = Clk;   
assign c3_p0_wr_clk = Clk;   
assign c3_p0_rd_clk = Clk;
   
// sdram signels
   
always @ (posedge Clk or negedge reset_n )
if (reset_n==1'b0)
  begin
    fifo_wr_en <=1'b0;
    c3_p0_cmd_en <= 1'b0;
    c3_p0_cmd_instr <= 3'b000;
    c3_p0_cmd_bl <=6'd0;
    c3_p0_cmd_byte_addr <= 30'h00000000;
    c3_p0_wr_en <= 1'b0;
    c3_p0_wr_mask <= 4'b0000;
    c3_p0_wr_data <= 32'h00000000;
    c3_p0_rd_en <= 1'b0;
  end
else
  if (waite_cnt==16'h0000)
  case (state)
    //-------------------------------------------------
    p_nop:
    begin
      fifo_wr_en <=1'b0;
      c3_p0_cmd_en <= 1'b0;
      c3_p0_cmd_instr <= 3'b000;
      c3_p0_cmd_bl <=6'd0;
      c3_p0_cmd_byte_addr <= c3_p0_cmd_byte_addr;
      c3_p0_wr_en <= 1'b0;
      c3_p0_wr_mask <= 4'b0000;
      c3_p0_wr_data <= 32'h00000000;
      c3_p0_rd_en <= 1'b0;
    end // case: p_nop
    
    //-------------------------------------------------
    p_rd_act:
    begin
      fifo_wr_en <=1'b0;
      c3_p0_cmd_en <= 1'b1;
      c3_p0_cmd_instr <= 3'b001;
      c3_p0_cmd_bl <=6'd31;
      c3_p0_cmd_byte_addr <= {2'h0,r_adr};
      c3_p0_wr_en <= 1'b0;
      c3_p0_wr_mask <= 4'b0000;
      c3_p0_wr_data <= 32'h00000000;
      c3_p0_rd_en <= 1'b0;
    end
    //-------------------------------------------------
    p_rd_wait:
    begin
      c3_p0_cmd_en <= 1'b0;
      c3_p0_cmd_instr <= 3'b000;
      c3_p0_cmd_bl <=6'd31;
      c3_p0_cmd_byte_addr <= c3_p0_cmd_byte_addr;
      c3_p0_wr_en <= 1'b0;
      c3_p0_wr_mask <= 4'b0000;
      c3_p0_wr_data <= 32'h00000000;
      c3_p0_rd_en <= 1'b0;
    end
    //-------------------------------------------------
    p_rd1:
    begin
      fifo_wr_en <=1'b0;
      c3_p0_cmd_en <= 1'b0;
      c3_p0_cmd_instr <= 3'b000;
      c3_p0_cmd_bl <=6'd31;
      c3_p0_cmd_byte_addr <= c3_p0_cmd_byte_addr;
      c3_p0_wr_en <= 1'b0;
      c3_p0_wr_mask <= 4'b0000;
      c3_p0_wr_data <= 32'h00000000;
      c3_p0_rd_en <= 1'b0;
    end 
    //-------------------------------------------------
    default:
    begin
      fifo_wr_en <=1'b0;
      c3_p0_cmd_en <= 1'b0;
      c3_p0_cmd_instr <= 3'b000;
      c3_p0_cmd_bl <=6'd0;
      c3_p0_cmd_byte_addr <= 30'h00000000;
      c3_p0_wr_en <= 1'b0;
      c3_p0_wr_mask <= 4'b0000;
      c3_p0_wr_data <= 32'h00000000;
      c3_p0_rd_en <= 1'b0;
    end
  endcase // case(state)
  else
    begin
      c3_p0_cmd_en <= 1'b0;
      c3_p0_cmd_byte_addr <= c3_p0_cmd_byte_addr;
      c3_p0_cmd_bl <=6'd0;
      c3_p0_cmd_instr <= 3'b000;
      c3_p0_wr_mask <= 4'b0000;
      c3_p0_cmd_bl <= c3_p0_cmd_bl;
      //if ((state==p_rd1)&&(waite_cnt>=t_rd_start)&&(waite_cnt<=t_rd_end))
      if ((state==p_rd1)&&(waite_cnt>=t_rd_start)&&(c3_p0_rd_count!=7'd0))
        begin   
          c3_p0_rd_en <= 1'b1;
        end
      else
        begin 
          c3_p0_rd_en <= 1'b0;
        end
      if ((state==p_rd1)&&(waite_cnt>=t_fifo_wr_start)&&(waite_cnt<=t_fifo_wr_end))
        begin   
          fifo_wr_en <= 1'b1;
        end
      else
        begin 
          fifo_wr_en <= 1'b0;
        end
    end
//daat generator


always @ (posedge pclk or negedge reset_n )
  if (reset_n==1'b0)
    begin
      c_cnt <= 3'b0; 
      dat_a_r <= 8'h00;
      dat_a_g <= 8'h00;
      dat_a_b <= 8'h00;
    end
  else
    if ((v_cnt==10'd1)&& (h_cnt==11'd39)) //vsync
      if(c_cnt==3'd0)
        if (dat_a_r==8'hff)
          begin
            c_cnt <= c_cnt + 3'd1;
            dat_a_r <= 8'hff;
          end
        else
          begin
            c_cnt <= c_cnt ;
            dat_a_r <= dat_a_r +8'h01;
          end
      else if(c_cnt==3'd1)
        if (dat_a_r==8'h00)
          begin
            c_cnt <= c_cnt + 3'd1;
            dat_a_r <= 8'h00;
          end
        else
          begin
            c_cnt <= c_cnt ;
            dat_a_r <= dat_a_r -8'h01;
          end
      else if(c_cnt==3'd2)
        if (dat_a_g==8'hff)
          begin
            c_cnt <= c_cnt + 3'd1;
            dat_a_g <= 8'hff;
          end
        else
          begin
            c_cnt <= c_cnt ;
            dat_a_g <= dat_a_g +8'h01;
          end
      else if(c_cnt==3'd3)
        if (dat_a_g==8'h00)
          begin
            c_cnt <= c_cnt + 3'd1;
            dat_a_g <= 8'h00;
          end
        else
          begin
            c_cnt <= c_cnt ;
            dat_a_g <= dat_a_g - 8'h01;
          end
      else if(c_cnt==3'd4)
        if (dat_a_b==8'hff)
          begin
            c_cnt <= c_cnt + 3'd1;
            dat_a_b <= 8'hff;
          end
        else
          begin
            c_cnt <= c_cnt ;
            dat_a_b <= dat_a_b +8'h01;
          end    
      else 
        if (dat_a_b==8'h00)
          begin
            c_cnt <= 3'd0;
            dat_a_b <= 8'h00;
          end
        else
          begin
            c_cnt <= c_cnt ;
            dat_a_b <= dat_a_b - 8'h01;
          end    
    else
      begin
        c_cnt <= c_cnt; 
        dat_a_r <= dat_a_r;
        dat_a_g <= dat_a_g;
        dat_a_b <= dat_a_b;
      end
//montor

assign mon = fifo_full ^ fifo_empty;
   

   
endmodule








