`timescale 1ns / 1ps

//*****************************************************************************
// File Name            : test_mb_mcs_sys.v
//-----------------------------------------------------------------------------
// Function             : MicroBlaze MCS system 
//                        
//-----------------------------------------------------------------------------
// Designer             : yokomizo 
//-----------------------------------------------------------------------------
// History
// -.-- 2013/07/10
//*****************************************************************************

module test_mb_mcs_sys;

        // Inputs
        reg Clk;
        reg Reset;
        reg UART_Rx;
        reg [3:0] dip_sw;

        // Outputs
        wire UART_Tx;
        wire [3:0] led;

        //DVI
        wire        DVI_TX0_P;
        wire        DVI_TX1_P;
        wire        DVI_TX2_P;
        wire        DVI_TXC_P;
        wire [12:0] mcb3_dram_a;
        wire [1:0] mcb3_dram_ba;
        wire mcb3_dram_cke;
        wire mcb3_dram_ras_n;
        wire mcb3_dram_cas_n;
        wire mcb3_dram_we_n;
        wire mcb3_dram_dm;
        wire mcb3_dram_udm;
        wire mcb3_dram_ck;
        wire mcb3_dram_ck_n;
        wire mon;

        // Bidirs
        wire [15:0] mcb3_dram_dq;
        wire mcb3_dram_udqs;
        wire mcb3_rzq;
        wire mcb3_dram_dqs;
   
        // Instantiate the Unit Under Test (UUT)
        mb_mcs_sys uut (
                .Clk(Clk), 
                .Reset(Reset), 
                .UART_Rx(UART_Rx), 
                .UART_Tx(UART_Tx), 
                .led(led), 
                .dip_sw(dip_sw),        
                .DVI_TX0_P(DVI_TX0_P),              // out   [TX] TX[0]-P (CML)
                .DVI_TX1_P(DVI_TX1_P),              // out   [TX] TX[1]-P (CML)
                .DVI_TX2_P(DVI_TX2_P),              // out   [TX] TX[2]-P (CML)
                .DVI_TXC_P(DVI_TXC_P),         
                .mcb3_dram_dq(mcb3_dram_dq), 
                .mcb3_dram_a(mcb3_dram_a), 
                .mcb3_dram_ba(mcb3_dram_ba), 
                .mcb3_dram_cke(mcb3_dram_cke), 
                .mcb3_dram_ras_n(mcb3_dram_ras_n), 
                .mcb3_dram_cas_n(mcb3_dram_cas_n), 
                .mcb3_dram_we_n(mcb3_dram_we_n), 
                .mcb3_dram_dm(mcb3_dram_dm), 
                .mcb3_dram_udqs(mcb3_dram_udqs), 
                .mcb3_rzq(mcb3_rzq), 
                .mcb3_dram_udm(mcb3_dram_udm), 
                .mcb3_dram_dqs(mcb3_dram_dqs), 
                .mcb3_dram_ck(mcb3_dram_ck), 
                .mcb3_dram_ck_n(mcb3_dram_ck_n), 
                .mon(mon)
        );

 
     lpddr_model_c3 u_mem3(
      .Dq    (mcb3_dram_dq),
      .Dqs   ({mcb3_dram_udqs,mcb3_dram_dqs}),
      .Addr  (mcb3_dram_a),
      .Ba    (mcb3_dram_ba),
      .Clk   (mcb3_dram_ck),
      .Clk_n (mcb3_dram_ck_n),
      .Cke   (mcb3_dram_cke),
      .Cs_n  (1'b0),
      .Ras_n (mcb3_dram_ras_n),
      .Cas_n (mcb3_dram_cas_n),
      .We_n  (mcb3_dram_we_n),
      .Dm    ({mcb3_dram_udm,mcb3_dram_dm})
      );
 
        initial begin
                // Initialize Inputs
                repeat(100000)begin
                  Clk = 0;
                  #5;
                  Clk = 1;
                  #5;
                  end
                $stop;
        end
 
 
        initial begin
                // Initialize Inputs
                Reset = 1;
                UART_Rx = 0;
                dip_sw = 0;

                // Wait 100 ns for global reset to finish
                #100;
                Reset = 0;        
                // Add stimulus here
        end
endmodule

